package be.iminds.ilabt.jfed.ui.cli2.instruction;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

/**
 * NewExperimentSpecification: specification of an experiment
 *
 * This uses either and ESpec or an RSpec. There are some common options.
 */
public class NewExperimentSpecification {
    public enum SourceType { ESPEC_FILE, ESPEC_DIR, ESPEC_GIT, ESPEC_URL, RSPEC_FILE, RSPEC_URL, RSPEC_DIRECT } //NOT RSPEC_GENERATED here, use ESpec for that

    @Nonnull private final SourceType sourceType;
    @Nonnull private final String source;

//run --rspec-source filename --rspec-bind-to URN --rspec-force-rewrite true --rspec-allow-stitching true

//    private static class ESpec {
//        private enum SourceType { ARCHIVE_FILE, DIR_FILE, URL }
//        private final SourceType sourceType;
//        private final String source;
//    }
//
//    private static class RSpec {
//        private enum SourceType { FILE, URL, DIRECT } //NOT generated here, use ESpec for that
//        private final SourceType sourceType;
//        private final String source;
//    }

    @Nullable private final String bindTo;
    private final boolean forceRewrite;
    private final boolean allowStitching;

    public NewExperimentSpecification(@JsonProperty("sourceType") @Nullable SourceType sourceType,
                                      @JsonProperty("source") @Nullable String source,
                                      @JsonProperty("bindTo") @Nullable String bindTo,
                                      @JsonProperty("forceRewrite") @Nullable Boolean forceRewrite,
                                      @JsonProperty("allowStitching") @Nullable Boolean allowStitching) {
        this.sourceType = sourceType == null ? new NewExperimentSpecification().sourceType : sourceType;
        this.source = source == null ? new NewExperimentSpecification().source : source;
        this.bindTo = bindTo == null ? new NewExperimentSpecification().bindTo : bindTo;
        this.forceRewrite = forceRewrite == null ? new NewExperimentSpecification().forceRewrite : forceRewrite;
        this.allowStitching = allowStitching == null ? new NewExperimentSpecification().allowStitching : allowStitching;
    }

    public NewExperimentSpecification() {
        this.sourceType = SourceType.RSPEC_DIRECT;
        this.source = "todo";
        this.bindTo = null;
        this.forceRewrite = false;
        this.allowStitching = false;
    }

    @JsonProperty
    @Nonnull
    public SourceType getSourceType() {
        return sourceType;
    }

    @JsonIgnore
    public boolean isESpec() {
        switch(sourceType) {
            case ESPEC_FILE:
            case ESPEC_URL:
            case ESPEC_DIR:
            case ESPEC_GIT:
                return true;
            case RSPEC_DIRECT:
            case RSPEC_FILE:
            case RSPEC_URL:
                return false;
            default: throw new RuntimeException("Unsupported sourceType: "+sourceType);
        }
    }

    @JsonProperty
    @Nonnull
    public String getSource() {
        return source;
    }

    @JsonProperty
    @Nullable
    public String getBindTo() {
        return bindTo;
    }

    @JsonProperty
    public boolean isForceRewrite() {
        return forceRewrite;
    }

    @JsonProperty
    public boolean isAllowStitching() {
        return allowStitching;
    }
}
